function [e] = AdslTEQ(h, Le, Lp, D)
% h  - DSL (channel) impulse response to be shortened
% Le - chosen length of the shortening filter - TEQ equalizer
% Lp - chosen length of the impulse response after shortening (P-prefix)
% D  - chosen delay of the shortened impulse response (in samples)
% e  - calculated weights of the TEQ equalizer

disp('Designing TEQ equalizer ...');
Lh = length(h); Lz = Lh+Le-1-D-Lp;         % dimensions

H = zeros(Le, Lh+Le-1);                    % convolutional channel matrix
for i=1:Le
   H(i,i:i+Lh-1) = h;
end

W1 = zeros(Lh+Le-1, Lh+Le-1);              % square matrix filled with 0s
W0 = zeros(Lh+Le-1, Lh+Le-1);              % square matrix filled with 0s 
rect=[zeros(1,D),ones(1,Lp),zeros(1,Lz)];% "rectangular" vector [ 001111000 ] 
W1 = diag(rect);   % matrix with 1s in the center of main diagonal, 0s outside
W0 = diag(1-rect); % matrix with 0s in the center of main diagonal, 1s outside
E1 = H*W1*H';                              % h(n) energy in the window
G1 = chol(E1);                             %
E0 = H*W0*H';                              % h(n) energy outside the window
T = inv(G1)'*E0*inv(G1);                   %
[V,D] = eig(T);                            % eigenvalue decomposition of T
LambdaMin = min(diag(D));                  % minimum eigenvalue
LambdaMinIdx = find(diag(D)==LambdaMin);   % its position
e = V(:,LambdaMinIdx)' * inv(G1)';         % final solution, weights of TEQ filter 
