% lab22_ex_ADCppm3.m
clear all; close all;

Kiter = 50;  N = 512;        % number of DMT symbols, samples per symbol
ADCppm = 100;                % assumed ADC ppm error
ppm = []; cfo = [];          % initialization

% Transmitter
QAM4 = [ 1+j, -1+j, 1-j, -1-j ];           % 4-QAM constellation: 00, 01, 10, 11
numbers = floor(4*(rand(N/2-1,1)-10*eps)); % random sequence of numbers 0,1,2,3
S0 = QAM4( numbers+1 );                    % random sequence of 4-QAM states
S1  = [ sqrt(2) S0 sqrt(2) conj( S0(end:-1:1) ) ];  % putting carriers in different states
x1 = real( sqrt(N/2) * ifft( S1 ) );                % generation of a real-value signal
x  = repmat( x1, 1, Kiter); Nx=N*Kiter;             % repeating it Kiter times without CP

% Channel
h = load('DSLine.dat')';                       % channel impulse response
y = conv(x ,h); y = y(1:Nx)/max(y);  Ny = Nx;  % signal passing through a channel h
yn = y + 0.003*randn(1,Ny);                    % optional addition of noise
figure
subplot(311); plot(x(1:2*N));  title('x(n)');
subplot(312); plot(y(1:2*N));  title('y(n)');
subplot(313); plot(yn(1:2*N)); title('yn(n)'); pause

% Receiver
% y=x; Ny=Nx;                              % for test
y = yn; Ny = length(yn);
Nppm = Ny + (ADCppm*1e-6)*Ny;              % wrong number of signal samples
y = interp1( [0:Ny-1], y(1,1:Ny), [0 : Ny/Nppm : (Ny-1)],'spline');
y = reshape(y(1:N*Kiter),N,Kiter);         % in columns consecutive RX signals
Y = fft( y ); Y = Y.';                     % exchanged with their spectra

% ADCppm estimation
Ksize = 30; kf = (-Ksize : Ksize);         % neighborhood around 0 Hz
ppm = []; cfo = [];                        % initialization
for k = 2 : Kiter-2                        % repeated estimation loop
   Y1 = Y(k,:);  Y2 = Y(k+1,:);            % choosing two consecutive spectra
   Y21 = Y2 .* conj(Y1);                   % symbols, should be the same 
   ang = fftshift( angle( Y21 ) );         % check possible phase increase
   angi = ang( (N/2+1)+kf );               % angles around 0Hz
   coef = polyfit(kf, angi, 1); ppm1=coef(1)/(2*pi); cfo1=coef(2); % pause
 % figure; plot( kf,angi,'bo-',kf,coef(2)+coef(1)*kf,'r-'); grid; title('Angle(n)'); pause
   ppm=[ppm, ppm1]; cfo=[cfo, cfo1];       % store results
end
figure; Koffs = round( round(Kiter/4) );
subplot(211); plot(ppm); title('ADCppm estim (k)');
subplot(212); plot(cfo); title('CFO    estim (k)'); pause
ppm=sort(ppm); ppm=mean( ppm(Koffs : end-Koffs+1) ), % ppm estimation
cfo=sort(cfo); cfo=mean( cfo(Koffs : end-Koffs+1) ), % cfo estimation
