% lab22_ex_dsl_idea.m
% Demonstration of Digital Subscriber Line idea
% Multi-carrier transmission, Discrete MultiTone (DMT) 
clear all; close all;

inputType = 1;                               % 1=signal, 2=spectrum
Kiter = 10;                                  % number of DMT symbols
N = 16; P = 3;                               % symbol and cyclic prefix lengths
nstd = 0;                                    % channel noise standard deviation
h = [ 1, 0.5, 0, -0.25 ]; Nh = length(h);    % channel impulse response
H = fft( [h zeros(1,N-Nh)] )/N; H = H.';     % channel frequency response
plot( 20*log10(abs(H)) ); title('|H(k)|');   % its figure
Nx = Kiter*(N+P);                            % transmitted signal length 
x = []; x1ref = []; S1ref = [];              % initialization
for k = 1 : Kiter                            % DMT signal generation
    if(inputType==1) x1 = randn(1,N);        % # real time-domain signal
    else                                     % @  
      S0 = randn(1,N/2-1) + j*randn(1,N/2-1);% @ half of the DFT spectrum
      S1  = [ 0 S0 0 conj( S0(end:-1:1) ) ]; % @ its Hermitian extension
      x1 = ifft( S1 );                       % @ real signal from IDFT
      S1ref = [ S1ref, S1.'];                % @ storing carrier states 
    end
    x1p = [ x1(1,N-P+1:N), x1 ];             % signal plus prefix of one DMT symbol
    x = [ x, x1p ];                          % chain of signals of many DMT symbols
    x1ref = [ x1ref,  x1' ];                 % storing transmitted signals as ref  
end                                          % end of signal generation loop
y = conv(x ,h); y = y(1:Nx);                 % signal is coming through the channel
y = y + nstd*randn(1,Nx);                    % channel is noisy, change nstd
y1p = reshape(y,N+P,Kiter);                  % in columns RX signals with prefixes
y1 = y1p( P+1:N+P,1:Kiter);                  % removing prefixes
Y1e = fft(y1)/N ./ repmat(H,1,Kiter);        % channel equalization
if(inputType==1)                             % FOR INPUT SIGNAL
   y1e = ifft( Y1e );                        %    going back to time domain
   error1 = max( max( abs( x1ref - y1e ))),  %    error in the receiver
else                                         % FOR INPUT SPECTRUM 
   error2 = max( max( abs( S1ref - Y1e ))),  %    error in freq domain
end