function [e] = myTEQ(h, Le, Lp, D)
% h  - DSL (channel) impulse response to be shortened
% Le - chosen length of the shortening filter  TEQ equalizer
% Lp - chosen length of the impulse response after shortening (P-prefix)
% D  - chosen delay of the shortened impulse response
% e  - calculated weights of the TEQ equalizer
% In Octave: >>pkg load communications[ENTER]
 
disp('Designing TEQ equlizer ...');

Lh = length(h); Lz = Lh+Le-1-D-Lp;
H = zeros(Le, Lh+Le-1);

for i=1:Le
   H(i,i:i+Lh-1) = h;                      % convolutional channel matrix
end
W1 = zeros(Lh+Le-1, Lh+Le-1);
W0 = zeros(Lh+Le-1, Lh+Le-1);
rect=[zeros(1,D),ones(1,Lp),zeros(1,Lz)];
W1 = diag(rect);                           % 1 in the centre
W0 = diag(1-rect);                         % 0 centre, 1 outside
E1=H*W1*H';                                % h(n) energy in the window
G1 = chol(E1);                             %
E0=H*W0*H';                                % h(n) energy outside window
T = inv(G1)'*E0*inv(G1);                   %
[V,D] = eig(T);                            % eigenvalue decomposition of T
LambdaMin = min(diag(D));                  % minimum eigenvalue
LambdaMinIdx = find(diag(D)==LambdaMin);   % its position
e = V(:,LambdaMinIdx)' * inv(G1)';         % final solution, weights of TEQ filter 

e = e/max(e);

H = convmtx(h',Le);                        % In Octave: >>pkg load communications[ENTER]

delay = 0;
%for delay = Dmin:Dmax % for each delay to be searched
   
  % Hwin: inside the window
  Hwin = H(delay+1:delay+Lp,:);
  % Hwall: outside the window
  Hwall = [H(1:delay,:); H(delay+Lp+1:size(H,1),:)];
  % energy of Hwall
  A = transpose(Hwall)*Hwall;
  % energy of Hwin
  B = transpose(Hwin)*Hwin;
 
[sqrtA] = chol(A);
  C = inv(sqrtA.')*B*inv(sqrtA);
% C = inv(sqrt(B))*A*inv(sqrt(B.'));
[V,D] = eig(C);
e1 = V(:,1);
e1 = inv(sqrt(B.'))*e1; e1 = real(e1);
e1 = e1/max(e1);

if(0)
  % Cholesky decomposition
  [sqrtA] = chol(A);
  % composite matrix
  C = inv(sqrtA.') * B * inv(sqrtA);
  [lambda q] = maxeig(C);
  w = inv(sqrtA) * q;
end

[V,D] = eig(B,A);    % B*e = lambda*A*e
e2 = V(:,end),
e2 = e1/max(e2);

[V,D] = eig(A);
e3 = V(:,1)
e3 = e3/max(e3),
size(e), size(e1), size(e3), pause
E = [ e' e1(1:16) e2(1:16) e3(1:16) ], pause
plot(E);
pause
